program boson

! *******************************************************************
! *                                                                 *
! * perform the quantum-mechanical complement to the classical step *
! * of the dielectric theory of eels in specular geometry using a   *
! * suitable thermodynamical average of the quantized surface       *
! * harmonic oscillators                                            *
! *                                                                 *
! *******************************************************************

  implicit none

  integer, parameter :: mmax=14, nmax=2**mmax
  double precision :: asym, emax, emin, gauss, t, width, wmin, wmax, y
  double precision :: p2(nmax)
  integer :: i, np, ioStatus
  character (len = 72) :: comment(2)
  double precision, allocatable :: xout(:), yout(:)
  integer :: nout
  logical :: debug

! *** read input parameters

  call change_working_dir()
  open(unit = 13, file = 'bosin')
! target temperature (Kelvin)
  read(13, *) t
! width of the instrumental response (cm**-1)
  read(13, *) width
! fraction of gaussian for the instrumental response
  read(13, *) gauss
! asymmetry of the instrumental response
  read(13, *) asym
! lower and upper energy losses for this computation (cm**-1)
  read(13, *) emin
  read(13, *) emax
  close(unit=13)

  write(*,*) 'program boson (September 2020)'
  write(*,'(a, f6.1, a, f7.2, a)') ' t =', t, ' K, width =', width, ' cm**-1'
  write(*,'(a, f5.2, a, f5.2)') ' gauss =', gauss, ', asym =', asym
  write(*,'(a, g11.4, a, g11.4, a)') ' energy losses from', emin, ' to', emax, ' cm**-1'

! *** read the table of values of the classical loss spectrum

  open(unit = 12, file = 'eelsou')
  read(12, '(a48)') comment(1)
  read(12, '(a72)') comment(2)
  np = 0
  do
    read(12, *, IOSTAT = ioStatus) wmax, y
    if (ioStatus /= 0) exit
    if (wmax < 0.0d0) cycle
    np = np + 1
    p2(np) = y
    if (np == 1) wmin = wmax
    if (np < nmax) cycle
  enddo
  close(unit = 12)

  if (np <= 0) stop '*** no input values for pcl ***'

  write(*,*) comment(2)
  write(*,'(a, i6, a, g15.7, a, g15.7)') ' read', np, ' values of pcl from', wmin, ' to', wmax

! length calculation for xout, yout.
!
  nout = 2**14
  allocate (xout(nout))
  allocate (yout(nout))

  debug = .false.

  call doboson(t, width, gauss, asym, emin, emax, &
               wmin, wmax, np, p2, debug, xout, yout, nout)

  open(unit = 14, file = 'bosou')
  write(14, '(a, a, f6.1, a, f5.2)') comment(1), 'T =', t, ' GAUSS =', gauss
  write(14, *) comment(2)
  do i = 1, nout
    write(14, '(2e15.7)') xout(i), yout(i)
  end do
  close(unit = 14)
  write(*,*) nout, ' values written on disk'

  deallocate (xout, yout)
  stop
end program boson
