module seteps_mod
contains
subroutine seteps(neps, nos, oscType, osc, epsinf, wn, eps, layers)

! ******************************************************************
! *                                                                *
! * set up long-wavelength dielectric functions of the layers for  *
! * the present frequency wn (in cm**-1)                           *
! *                                                                *
! * neps:           number of epsilon values                       *
! * nos:            number of oscillators                          *
! * oscType:        oscillator type: Lorentz, Kurosawa, drude      *
! * osc:            oscillator parameters, wTO, Q, lambda          *
! * epsinf:         epsilon at infinite frequency                  *
! * wn:             frequency                                      *
! * eps:            epsilon                                        *
! * layers:         number of layers                               *
! *                                                                *
! ******************************************************************

  use oscillator_mod

  integer, intent(in) :: neps, nos(:), oscType(:)
  double precision, intent(in) :: osc(:, :), epsinf(:), wn
  double complex, intent(in out) :: eps(:)
  integer, intent(in) :: layers

  integer :: i, j, k
  double complex :: one, nomi, deno, addLorentz, addDrude
  double precision :: wn2, b, gamma

  one = dcmplx(1.0, 0.0)
  wn2 = wn**2
  k = 1
  do i = 1, neps
    nomi = one
    deno = one
    addLorentz = dcmplx(0.0, 0.0)
    addDrude = dcmplx(0.0, 0.0)
    do j = 1, nos(i)
      select case (oscType(k))
        case (Lorentz) ! osc3 is lambda and not gammma
          gamma = osc(1,k) * osc(3,k)
          addLorentz = addLorentz + osc(1,k)**2 * osc(2,k) / dcmplx(osc(1,k)**2 - wn2, -wn * gamma)
        case (Kurosawa)
          b = wn / osc(1,k)
          nomi = nomi *  osc(3,k)**2 * (1.0 - b * dcmplx( b, osc(4,k)/osc(3,k)) )
          deno = deno * (osc(1,k)**2 -       wn * dcmplx(wn, osc(2,k)         ) )
        case (Drude)            ! As with lorentz, osc2 is lambda and not gammma
          gamma = osc(1,k) * osc(2,k)
          addDrude = addDrude - osc(1,k)**2 / dcmplx(wn2, wn * gamma)
        case (extendedDrude)    ! osc2 and osc3 are gamma_Plasma and Gamma_0
          addDrude = addDrude - dcmplx(osc(1,k)**2, wn*(osc(2,k)-osc(3,k))) / dcmplx(wn2, wn * osc(2,k))
        case default
          write (*,'(A)') '*** Error in oscillator type! ***'
      end select
      eps(i) = epsinf(i) * (nomi / deno) + addDrude + addLorentz
      k = k + 1
    enddo
  enddo

  if (neps == layers + 1) then
! the substrate is a anisotropic uniaxial material
    eps(layers) = cdsqrt(eps(layers) * eps(layers + 1))
    if (dimag(eps(layers)) < 0.0d0) then
      eps(layers) = -eps(layers)
    endif
  endif
  return
end subroutine seteps
end module seteps_mod
